(* basic_control.ml : demonstrations of various control structures
   like for/do and if/else *)

open Printf;;

(**********************************************************************************)
(* LOOPING with for/do *)

(* print the first n even numbers *)
let print_evens1 n =
  for i=0 to n-1 do               (* loop increment by 1 each iter *)
    let e = 2*i in                (* local let *)
    printf "%d : %d\n" i e;       (* last statement, semicolon is optional *)
  done;                           (* end of scope for i *)
;;


(* print first n even numbers, descending order *)
let print_evens_descend n =
  for i=n-1 downto 0 do
    let e = 2*i in                (* local let *)
    printf "%d : %d\n" i e;       (* last statement, semicolon is optional *)
  done;                           (* end of scope for i *)
;;

(**********************************************************************************)
(* CONDITIONALS with if/else *)

let is_even n =
  if n mod 2 = 0 then             (* mod is remainder operator *)
    true                          (* return true *)
  else
    false                         (* return false *)
;;

(* print a message only if even *)
let print_if_even n =
  if is_even n then
    printf "%d is even\n" n;      (* no associated else case *)
;;

(* form a string based on even/oddness *)
let even_odd_str1 n =             (* standard style *)
  if n mod 2 = 0 then             (* condition with *)
    let nstr = string_of_int n in (* differing assignments *)
    let msg = " is even" in
    nstr^msg                      (* consequent return val of function *)
  else
    let nstr = string_of_int n in
    let msg = " is odd" in
    nstr^msg                      (* alternate return val of function *)
;;

(* form a string based on even/oddness *)
let even_odd_str2 n =             (* more functional style *)
  let nstr = string_of_int n in   (* unconditional binding *)
  let msg =                       (* bind this value.. *)
    if n mod 2 = 0 then           (* based on this condition *)
      " is even"                  (* condition true *)
    else
      " is odd"                   (* condition false *)
  in
  nstr^msg                        (* return value of function *)
;;



(**********************************************************************************)
(* MIXED LOOPS CONDITIONALS *)

(* print first n even numbers *)
let print_evens2 n =
  for i=0 to 2*n-1 do
    if i mod 2 = 0 then           (* if but no else *)
      let index = i / 2 in        (* local let *)
      printf "%d : %d\n" index i  (* last statement in loop, no need for semicolon *)
  done;
;;

(* print both even and odds labeled as such *)
let print_both1 n =               (* conventional definition *)
  for i=1 to n do                 
    if is_even i then             (* if statement chooses *)
      printf "%d : even\n" i      (* one printf or *)
    else
      printf "%d : odd\n" i       (* another printf *)
  done;
;;
    
(* print both even and odds labeled as such *)
let print_both2 n =
  for i=1 to n do                 (* more functional style *)
    let msg =                     (* bind msg to one of... *)
      if is_even i then           (* if statement chooses *)
        "even"                    (* one message *)
      else
        "odd"                     (* or another *)
    in
    printf "%d : %s\n" i msg;     (* single printf *)
  done;
;;

